//
// Copyright (c) ZeroC, Inc. All rights reserved.
//

#include <Ice/ResponseHandler.h>

using namespace std;
using namespace IceInternal;

#ifndef ICE_CPP11_MAPPING
IceUtil::Shared* IceInternal::upCast(ResponseHandler* obj) { return obj; }
#endif
