//
// Copyright (c) ZeroC, Inc. All rights reserved.
//

#ifndef ICE_REGISTER_PLUGINS_INIT_H
#define ICE_REGISTER_PLUGINS_INIT_H

namespace IceInternal
{

class RegisterPluginsInit
{
public:

    RegisterPluginsInit();
};

}

#endif
